# Copyright (C) 2025 Belaid Ziane

# ***** BEGIN GPL LICENSE BLOCK *****
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# ***** END GPL LICENSE BLOCK *****

bl_info = {
    "name": "Measure and Scale",
    "author": "Bill3D(Belaid Ziane)",
    "version": (1,0.0),
    "blender": (4, 2, 0),
    "location": "View3D > UI > Npanel > Item > measure and scale",
    "description": "Interactively measure between two vertices, scale the object uniformly to a target dimension.",
    "category": "Object",

}
import bpy
import bmesh
import gpu
import blf
from mathutils import Vector, Matrix
from bpy.types import Operator, Panel, PropertyGroup, Scene # Added Scene
from bpy.props import FloatProperty, StringProperty, PointerProperty, EnumProperty # Added EnumProperty
import bpy_extras.view3d_utils
from gpu_extras.batch import batch_for_shader

# --- Unit Conversion Constants ---
METERS_TO_CM = 100.0
METERS_TO_MM = 1000.0
METERS_TO_FEET = 3.28084
METERS_TO_INCHES = 39.3701
FEET_TO_INCHES = 12.0

# --- Property Group for Settings ---
class ScaleInteractiveSettings(PropertyGroup):
    metric_unit: EnumProperty(
        name="Metric Unit",
        items=[
            ('M', "m", "Meters"),
            ('CM', "cm", "Centimeters"),
            ('MM', "mm", "Millimeters"),
        ],
        default='M',
        description="Select the metric unit for display"
    )
    system_unit: EnumProperty(
        name="System",
        items=[
            ('METRIC', "Metric", "Use Metric units (m, cm, mm)"),
            ('IMPERIAL_FT', "ft", "Use Imperial units (Feet)"),
            ('IMPERIAL_IN', "in", "Use Imperial units (Inches)"),
        ],
        default='METRIC',
        description="Select the unit system for display"
    )

# --- Unit Conversion Functions ---
def get_unit_suffix(settings):
    """Returns the appropriate unit suffix string based on settings."""
    if settings.system_unit == 'METRIC':
        return settings.metric_unit.lower()
    elif settings.system_unit == 'IMPERIAL_FT':
        return 'ft'
    elif settings.system_unit == 'IMPERIAL_IN':
        return 'in'
    return '' # Default/fallback

def convert_to_display_unit(value_meters, settings):
    """Converts a value from meters to the selected display unit."""
    if settings.system_unit == 'METRIC':
        if settings.metric_unit == 'CM':
            return value_meters * METERS_TO_CM
        elif settings.metric_unit == 'MM':
            return value_meters * METERS_TO_MM
        else: # 'M'
            return value_meters
    elif settings.system_unit == 'IMPERIAL_FT':
        return value_meters * METERS_TO_FEET
    elif settings.system_unit == 'IMPERIAL_IN':
        return value_meters * METERS_TO_INCHES
    return value_meters # Fallback

def convert_from_display_unit(value_display, settings):
    """Converts a value from the selected display unit back to meters."""
    if settings.system_unit == 'METRIC':
        if settings.metric_unit == 'CM':
            return value_display / METERS_TO_CM
        elif settings.metric_unit == 'MM':
            return value_display / METERS_TO_MM
        else: # 'M'
            return value_display
    elif settings.system_unit == 'IMPERIAL_FT':
        return value_display / METERS_TO_FEET
    elif settings.system_unit == 'IMPERIAL_IN':
        return value_display / METERS_TO_INCHES
    return value_display # Fallback


MAX_SNAP_DISTANCE_SCREEN = 25.0
MAX_3D_SNAP_VALIDATION_DIST_SQ = 0.5**2
SNAP_POINT_SIZE = 10.0
modal_data = {
    "draw_handle": None,
    "shader": None,
    "start_vertex_co": None,
    "end_vertex_co": None,
    "current_mouse_pos": None,
    "hover_vertex_co": None,
    "object_name": None,
    "original_mode": None,
    "measured_distance": 0.0,
    "status_message": "Click first vertex",
    "original_wireframe_state": None
}
def ray_cast(context, position):
    """Raycast from screen position into 3D space"""
    region = context.region
    region3D = context.space_data.region_3d
    if not region or not region3D:
        return False, None, None, -1, None, None, None
    view_point = bpy_extras.view3d_utils.region_2d_to_origin_3d(region, region3D, position)
    view_vector = bpy_extras.view3d_utils.region_2d_to_vector_3d(region, region3D, position)
    depsgraph = context.evaluated_depsgraph_get()
    result, location, normal, index, object_hit, matrix = \
        context.scene.ray_cast(depsgraph, view_point, view_vector)
    return result, location, normal, index, object_hit, matrix, view_point
def find_nearest_vertex_world(context, event, obj):
    """Find nearest visible vertex using improved snapping logic"""
    mouse_pos = Vector((event.mouse_region_x, event.mouse_region_y))
    region = context.region
    rv3d = context.space_data.region_3d
    if not region or not rv3d:
        return None
    ray_result, ray_hit_loc, _, _, hit_obj, _, _ = ray_cast(context, mouse_pos)
    if not ray_result or hit_obj != obj:
        return None
    min_dist = float('inf')
    nearest_vert_co = None
    obj_eval = obj.evaluated_get(context.evaluated_depsgraph_get())
    try:
        mesh_eval = obj_eval.to_mesh()
    except RuntimeError:
        if hasattr(obj_eval, 'to_mesh_clear'):
             obj_eval.to_mesh_clear()
        return None
    if not mesh_eval or not mesh_eval.vertices:
        if mesh_eval and hasattr(obj_eval, 'to_mesh_clear'):
             obj_eval.to_mesh_clear()
        return None
    world_matrix = obj.matrix_world
    for v in mesh_eval.vertices:
        world_co = world_matrix @ v.co
        if (world_co - ray_hit_loc).length_squared > MAX_3D_SNAP_VALIDATION_DIST_SQ:
            continue
        try:
            screen_co = bpy_extras.view3d_utils.location_3d_to_region_2d(region, rv3d, world_co)
        except (TypeError, ReferenceError):
            screen_co = None
        if screen_co:
            dist = (screen_co - mouse_pos).length
            if dist < MAX_SNAP_DISTANCE_SCREEN and dist < min_dist:
                min_dist = dist
                nearest_vert_co = world_co
    if hasattr(obj_eval, 'to_mesh_clear'):
         obj_eval.to_mesh_clear()
    return nearest_vert_co
def draw_callback_px(context):
    if not context.area:
        return
    if not modal_data or "status_message" not in modal_data:
        handle = modal_data.get("draw_handle") if modal_data else None
        if handle:
            try:
                bpy.types.SpaceView3D.draw_handler_remove(handle, 'WINDOW')
            except Exception:
                pass
        if modal_data: modal_data.clear()
        return

    settings = context.scene.scale_interactive_settings # Get unit settings
    font_id = 0
    blf.size(font_id, 14)
    start_co = modal_data.get("start_vertex_co")
    end_co = modal_data.get("end_vertex_co")
    hover_co = modal_data.get("hover_vertex_co")
    mouse_pos = modal_data.get("current_mouse_pos")
    screen_start_co, screen_end_co, screen_hover_co = None, None, None
    rv3d = context.space_data.region_3d if context.space_data else None
    if not rv3d: return
    if start_co:
        try:
            screen_start_co = bpy_extras.view3d_utils.location_3d_to_region_2d(
                context.region, rv3d, start_co)
        except (TypeError, ReferenceError, AttributeError):
            pass
    if end_co:
        try:
            screen_end_co = bpy_extras.view3d_utils.location_3d_to_region_2d(
                context.region, rv3d, end_co)
        except (TypeError, ReferenceError, AttributeError):
            pass
    if hover_co and not end_co:
        try:
            screen_hover_co = bpy_extras.view3d_utils.location_3d_to_region_2d(
                context.region, rv3d, hover_co)
        except (TypeError, ReferenceError, AttributeError):
            pass
    shader = gpu.shader.from_builtin('UNIFORM_COLOR')
    shader.bind()
    if screen_hover_co:
        shader.uniform_float("color", (1.0, 0.5, 0.0, 1.0))
        gpu.state.point_size_set(SNAP_POINT_SIZE)
        batch = batch_for_shader(shader, 'POINTS', {"pos": [screen_hover_co]})
        batch.draw(shader)
    if screen_start_co:
        shader.uniform_float("color", (0.0, 1.0, 0.0, 1.0))
        gpu.state.point_size_set(SNAP_POINT_SIZE + 2)
        batch = batch_for_shader(shader, 'POINTS', {"pos": [screen_start_co]})
        batch.draw(shader)
    if screen_end_co:
        shader.uniform_float("color", (1.0, 0.0, 0.0, 1.0))
        gpu.state.point_size_set(SNAP_POINT_SIZE + 2)
        batch = batch_for_shader(shader, 'POINTS', {"pos": [screen_end_co]})
        batch.draw(shader)
    gpu.state.point_size_set(1)
    line_drawn = False
    if screen_start_co:
        if screen_end_co:
            shader.uniform_float("color", (0.53, 0.81, 0.92, 1.0))
            gpu.state.line_width_set(3.0)
            gpu.state.blend_set('ALPHA')
            batch = batch_for_shader(shader, 'LINES', {"pos": [screen_start_co, screen_end_co]})
            batch.draw(shader)
            line_drawn = True
            measured_distance = modal_data.get("measured_distance", 0.0)
            if measured_distance > 0.0001:
                text_pos = (Vector(screen_start_co) + Vector(screen_end_co)) / 2.0
                # Convert and display measured distance
                display_distance = convert_to_display_unit(measured_distance, settings)
                unit_suffix = get_unit_suffix(settings)
                blf.position(font_id, text_pos.x + 10, text_pos.y + 10, 0)
                blf.color(font_id, 1.0, 1.0, 1.0, 1.0)
                fixed_precision = 4
                blf.draw(font_id, f"{display_distance:.{fixed_precision}f} {unit_suffix}")
        elif mouse_pos:
            target_screen_pos = mouse_pos
            dynamic_distance = 0.0
            if screen_hover_co:
                target_screen_pos = screen_hover_co
                if hover_co:
                    dynamic_distance = (hover_co - start_co).length
            shader.uniform_float("color", (0.8, 0.8, 0.8, 0.9))
            gpu.state.line_width_set(2.0)
            gpu.state.blend_set('ALPHA')
            batch = batch_for_shader(shader, 'LINES', {"pos": [screen_start_co, target_screen_pos]})
            batch.draw(shader)
            line_drawn = True
            if dynamic_distance > 0.0001:
                text_pos = (Vector(screen_start_co) + Vector(target_screen_pos)) / 2.0
                # Convert and display dynamic distance
                display_dynamic_distance = convert_to_display_unit(dynamic_distance, settings)
                unit_suffix = get_unit_suffix(settings)
                blf.position(font_id, text_pos.x + 10, text_pos.y + 10, 0)
                blf.color(font_id, 1.0, 1.0, 1.0, 1.0)
                fixed_precision = 4
                blf.draw(font_id, f"{display_dynamic_distance:.{fixed_precision}f} {unit_suffix}")
    if line_drawn:
        gpu.state.blend_set('NONE')
        gpu.state.line_width_set(1.0)
    blf.position(font_id, 20, 30, 0)
    blf.color(font_id, 1.0, 1.0, 1.0, 1.0)
    blf.draw(font_id, modal_data.get("status_message", "Initializing..."))
class OBJECT_OT_ScaleToDimensionInteractive(Operator):
    bl_idname = "object.scale_to_dimension_interactive"
    bl_label = "Scale to Dimension Interactive"
    bl_options = {'REGISTER', 'UNDO'}
    @classmethod
    def poll(cls, context):
        return context.active_object is not None and context.area.type == 'VIEW_3D'
    def invoke(self, context, event):
        if context.area.type != 'VIEW_3D':
            self.report({'WARNING'}, "Active space must be a 3D View")
            return {'CANCELLED'}
        if modal_data.get("draw_handle") is not None:
             if modal_data.get("status_message", "").startswith("Dialog Open"):
                 self.report({'INFO'}, "Scale dialog is already open.")
             else:
                 self.report({'WARNING'}, "Interactive scale operation already in progress.")
             return {'CANCELLED'}
        active_obj = context.active_object
        if not active_obj:
            self.report({'WARNING'}, "No active object selected")
            return {'CANCELLED'}
        if active_obj.type != 'MESH':
            self.report({'WARNING'}, "Active object must be a Mesh")
            return {'CANCELLED'}
        space_data = context.space_data
        original_wireframe_state = None
        wireframe_changed = False
        if space_data and hasattr(space_data, 'overlay'):
            try:
                original_wireframe_state = space_data.overlay.show_wireframes
                if not original_wireframe_state:
                    space_data.overlay.show_wireframes = True
                    wireframe_changed = True
            except AttributeError:
                 print("Warning: Could not access space_data.overlay to manage wireframe.")
                 original_wireframe_state = None
            except Exception as e:
                 print(f"Error accessing wireframe state: {e}")
                 original_wireframe_state = None
        else:
            print("Warning: context.space_data not available for wireframe management.")
        modal_data.clear()
        modal_data["start_vertex_co"] = None
        modal_data["end_vertex_co"] = None
        modal_data["measured_distance"] = 0.0
        modal_data["object_name"] = active_obj.name
        modal_data["original_mode"] = context.mode
        modal_data["status_message"] = "Click first vertex (ESC to cancel)"
        modal_data["draw_handle"] = None
        modal_data["shader"] = gpu.shader.from_builtin('UNIFORM_COLOR')
        modal_data["original_wireframe_state"] = original_wireframe_state
        args = (context,)
        try:
            draw_handle = bpy.types.SpaceView3D.draw_handler_add(
                draw_callback_px, args, 'WINDOW', 'POST_PIXEL')
            modal_data["draw_handle"] = draw_handle
        except Exception as e:
            self.report({'ERROR'}, f"Failed to add draw handler: {e}")
            self._restore_wireframe(context, original_wireframe_state, wireframe_changed)
            return {'CANCELLED'}
        context.window_manager.modal_handler_add(self)
        context.area.tag_redraw()
        return {'RUNNING_MODAL'}
    def modal(self, context, event):
        if modal_data.get("draw_handle") is None and "object_name" not in modal_data:
            self._remove_handle_and_data(context)
            return {'CANCELLED'}
        if not context.area:
            return self.cancel(context)
        context.area.tag_redraw()
        obj = context.scene.objects.get(modal_data.get("object_name"))
        if not obj:
            self.report({'WARNING'}, "Original object lost!")
            return self.cancel(context)
        if obj.type != 'MESH':
            self.report({'WARNING'}, "Object is no longer a mesh.")
            return self.cancel(context)
        if event.type == 'MOUSEMOVE':
            if modal_data.get("end_vertex_co") is None:
                modal_data["current_mouse_pos"] = (event.mouse_region_x, event.mouse_region_y)
                modal_data["hover_vertex_co"] = find_nearest_vertex_world(context, event, obj)
            else:
                modal_data["current_mouse_pos"] = None
                modal_data["hover_vertex_co"] = None
        elif event.type == 'LEFTMOUSE' and event.value == 'PRESS':
            if modal_data.get("end_vertex_co") is None:
                click_co = find_nearest_vertex_world(context, event, obj)
                if click_co:
                    if not modal_data.get("start_vertex_co"):
                        modal_data["start_vertex_co"] = click_co.copy()
                        modal_data["status_message"] = "Click second vertex (ESC to cancel)"
                        modal_data["hover_vertex_co"] = None
                    else:
                        if (click_co - modal_data["start_vertex_co"]).length < 0.0001:
                            self.report({'INFO'}, "Second point too close to the first.")
                            return {'RUNNING_MODAL'}
                        modal_data["end_vertex_co"] = click_co.copy()
                        modal_data["measured_distance"] = (
                            modal_data["end_vertex_co"] - modal_data["start_vertex_co"]).length
                        modal_data["status_message"] = "Dialog Open - Enter Target Dimension"
                        modal_data["current_mouse_pos"] = None
                        modal_data["hover_vertex_co"] = None
                        context.area.tag_redraw()
                        bpy.ops.object.scale_confirm_dialog('INVOKE_DEFAULT',
                            measured_distance=modal_data["measured_distance"],
                            object_name=modal_data["object_name"],
                            original_mode=modal_data["original_mode"])
                        return {'FINISHED'}
                else:
                    self.report({'INFO'}, "No vertex found near cursor.")
                    return {'RUNNING_MODAL'}
        elif event.type in {'RIGHTMOUSE', 'ESC'}:
            return self.cancel(context)
        elif event.type in {'MIDDLEMOUSE', 'WHEELUPMOUSE', 'WHEELDOWNMOUSE'} or \
             (event.type == 'MOUSEMOVE' and event.alt and not event.ctrl and not event.shift) or \
             event.type in {'ROTATE', 'MOVE', 'ZOOM'} or \
             (event.type == 'TIMER'):
            return {'PASS_THROUGH'}
        return {'RUNNING_MODAL'}
    def finish(self, context):
        return {'FINISHED'}
    def cancel(self, context):
        self._remove_handle_and_data(context)
        if context.area:
            context.area.tag_redraw()
        self.report({'INFO'}, "Interactive scaling cancelled.")
        return {'CANCELLED'}
    def _restore_wireframe(self, context, original_state, wireframe_changed):
        """Helper to restore wireframe if needed."""
        if wireframe_changed and original_state is False:
            if context and context.space_data and hasattr(context.space_data, 'overlay'):
                try:
                    context.space_data.overlay.show_wireframes = original_state
                except Exception as e:
                    print(f"Info: Error restoring wireframe state during cleanup: {e}")
            else:
                 print("Info: Could not restore wireframe - context/space_data unavailable.")
    def _remove_handle_and_data(self, context):
        """Clean up draw handler, restore wireframe, and clear data."""
        handle = modal_data.get("draw_handle")
        if handle is not None:
            try:
                bpy.types.SpaceView3D.draw_handler_remove(handle, 'WINDOW')
            except ValueError: pass
            except Exception as e:
                print(f"Info: Error removing draw handler during cancel: {e}")
        original_state = modal_data.get("original_wireframe_state")
        wireframe_was_toggled_on = original_state == False
        self._restore_wireframe(context, original_state, wireframe_was_toggled_on)
        modal_data.clear()
class OBJECT_OT_ScaleConfirmDialog(Operator):
    bl_idname = "object.scale_confirm_dialog"
    bl_label = "Set Target Dimension"
    bl_options = {'REGISTER', 'INTERNAL'}
    measured_distance: FloatProperty(name="Measured Distance", default=0.0, options={'HIDDEN'})
    object_name: StringProperty(name="Object Name", default="", options={'HIDDEN'})
    original_mode: StringProperty(name="Original Mode", default="OBJECT", options={'HIDDEN'})
    target_dimension: FloatProperty(
        name="Target Dimension",
        description="Enter the desired dimension",
        default=1.0,
        min=0.00001,
        soft_min=0.01
        # Removed unit='LENGTH' to allow custom suffix in draw method
    )
    def _cleanup(self, context):
        """Unified cleanup for the dialog."""
        handle = modal_data.get("draw_handle")
        if handle is not None:
            try:
                bpy.types.SpaceView3D.draw_handler_remove(handle, 'WINDOW')
            except ValueError: pass
            except Exception as e:
                print(f"Info: Dialog failed to remove draw handler {handle}: {e}")
        if modal_data:
            original_state = modal_data.get("original_wireframe_state")
            wireframe_was_toggled_on = original_state == False
            if wireframe_was_toggled_on and original_state is False:
                if context and context.space_data and hasattr(context.space_data, 'overlay'):
                    try:
                        context.space_data.overlay.show_wireframes = original_state
                    except Exception as e:
                        print(f"Info: Dialog failed to restore wireframe state: {e}")
                else:
                    print("Info: Dialog could not restore wireframe - context/space_data unavailable.")
        modal_data.clear()
        if context and context.area:
            context.area.tag_redraw()
    def invoke(self, context, event):
        if not modal_data.get("draw_handle") or not modal_data.get("object_name"):
            self.report({'ERROR'}, "Cannot open dialog: Modal data is missing. Please restart the operation.")
            return {'CANCELLED'}

        settings = context.scene.scale_interactive_settings
        if self.measured_distance > 0.0001:
            # Set default target dimension in the current display unit
            self.target_dimension = convert_to_display_unit(self.measured_distance, settings)
        else:
            self.target_dimension = 1.0 # Default if no measurement

        modal_data["status_message"] = "Enter Target Dimension in Dialog (ESC to Cancel)"
        if context.area: context.area.tag_redraw()
        wm = context.window_manager
        return wm.invoke_props_dialog(self, width=300)
    def draw(self, context):
        layout = self.layout
        settings = context.scene.scale_interactive_settings
        col = layout.column()

        # Display Measured Distance in selected units
        row = col.row()
        row.label(text="Measured Distance:")
        fixed_precision = 4
        display_measured_dist = convert_to_display_unit(self.measured_distance, settings)
        unit_suffix = get_unit_suffix(settings)
        row.label(text=f"{display_measured_dist:.{fixed_precision}f} {unit_suffix}")

        # Target Dimension Input (label includes unit)
        col.prop(self, "target_dimension", text=f"Target Dimension ({unit_suffix})")

    def execute(self, context):
        settings = context.scene.scale_interactive_settings # Get settings
        obj_name = self.object_name
        settings = context.scene.scale_interactive_settings # Get settings
        obj_name = self.object_name
        orig_mode = self.original_mode
        measured_dist_meters = self.measured_distance # Already in meters
        target_dim_display = self.target_dimension # User input in display units
        self._cleanup(context)

        if measured_dist_meters <= 0.000001:
            self.report({'ERROR'}, "Invalid measured distance.")
            return {'CANCELLED'}
        if target_dim_display <= 0.000001:
            self.report({'ERROR'}, "Invalid target dimension.")
            return {'CANCELLED'}

        # Convert target dimension from display units back to meters
        target_dim_meters = convert_from_display_unit(target_dim_display, settings)
        if target_dim_meters <= 0.000001: # Check again after conversion
             self.report({'ERROR'}, "Target dimension too small after unit conversion.")
             return {'CANCELLED'}

        obj = context.scene.objects.get(obj_name)
        if not obj:
            self.report({'ERROR'}, f"Object '{obj_name}' not found.")
            return {'CANCELLED'}
        if obj.type != 'MESH':
            self.report({'ERROR'}, f"Object '{obj_name}' is not a Mesh.")
            return {'CANCELLED'}

        # Calculate scaling factor using meter values
        scaling_factor = target_dim_meters / measured_dist_meters
        current_mode = context.mode
        needs_mode_change = False
        original_active = context.view_layer.objects.active
        original_selection_names = [o.name for o in context.selected_objects if o]
        if current_mode != 'OBJECT':
            needs_mode_change = True
            try:
                bpy.ops.object.mode_set(mode='OBJECT')
            except RuntimeError as e:
                self.report({'ERROR'}, f"Failed to switch to Object Mode: {e}")
                return {'CANCELLED'}
        bpy.ops.object.select_all(action='DESELECT')
        obj.select_set(True)
        context.view_layer.objects.active = obj
        try:
            obj.scale *= scaling_factor
            bpy.ops.object.transform_apply(location=False, rotation=False, scale=True, properties=False)
        except Exception as e:
            self.report({'ERROR'}, f"Scaling failed: {e}")
            obj.select_set(False)
            for name in original_selection_names:
                sel_obj = context.scene.objects.get(name)
                if sel_obj: sel_obj.select_set(True)
            if original_active and original_active.name in context.scene.objects:
                context.view_layer.objects.active = original_active
            if needs_mode_change and context.view_layer.objects.active and orig_mode != 'OBJECT':
                 try: bpy.ops.object.mode_set(mode='EDIT' if orig_mode == 'EDIT_MESH' else orig_mode)
                 except: pass
            return {'CANCELLED'}
        if obj.name not in original_selection_names:
            obj.select_set(False)
        for name in original_selection_names:
            sel_obj = context.scene.objects.get(name)
            if sel_obj and sel_obj != obj :
                 sel_obj.select_set(True)
            elif sel_obj == obj and obj.name in original_selection_names:
                obj.select_set(True)
        restored_active = False
        if original_active and original_active.name in context.scene.objects:
            context.view_layer.objects.active = original_active
            restored_active = True
        elif obj.name in original_selection_names and obj.name in context.scene.objects:
             context.view_layer.objects.active = obj
             restored_active = True
        if needs_mode_change and orig_mode != 'OBJECT':
            current_active_after_restore = context.view_layer.objects.active
            if current_active_after_restore:
                try:
                    mode_to_set = 'EDIT' if orig_mode == 'EDIT_MESH' else orig_mode
                    bpy.ops.object.mode_set(mode=mode_to_set)
                except RuntimeError as e:
                    print(f"Could not switch object '{current_active_after_restore.name}' back to {orig_mode}: {e}")

        # Report using the display unit
        unit_suffix = get_unit_suffix(settings)
        self.report({'INFO'}, f"Object '{obj.name}' scaled by {scaling_factor:.4f}. New dimension: {target_dim_display:.4f} {unit_suffix}")
        return {'FINISHED'}
    def cancel(self, context):
        self._cleanup(context)
        self.report({'INFO'}, "Scaling cancelled.")
        return {'CANCELLED'}
class VIEW3D_PT_ScaleToDimensionInteractivePanel(Panel):
    bl_label = "Measure and Scale"
    bl_description = "Interactively measure and scale to a target dimension"
    bl_idname = "VIEW3D_PT_scale_to_dimension_interactive"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = 'Item' # Changed from 'Item' to 'Scale Tool' for better organization
    def draw(self, context):
        layout = self.layout
        scene = context.scene
        settings = scene.scale_interactive_settings # Access settings from scene

        col = layout.column(align=True)

        # --- Unit Selection ---
        box = col.box()
        box.label(text="Display Units:")
        row = box.row(align=True)
        row.prop(settings, "system_unit", expand=True)

        # Show metric options only if Metric system is selected
        if settings.system_unit == 'METRIC':
            row = box.row(align=True)
            row.prop(settings, "metric_unit", expand=True)

        col.separator()

        # --- Operator Button ---
        obj = context.active_object
        can_run = obj and obj.type == 'MESH'
        op_row = col.row()
        op_row.enabled = can_run and (modal_data.get("draw_handle") is None)
        op_row.operator(
            OBJECT_OT_ScaleToDimensionInteractive.bl_idname,
            text="Measure and Scale",
            icon='ARROW_LEFTRIGHT')
        if not can_run:
            col.label(text="Select a Mesh object.", icon='INFO')
        elif modal_data.get("draw_handle") is not None:
             status = modal_data.get("status_message", "Measuring...")
             if status.startswith("Dialog Open") or status.startswith("Enter Target"):
                 col.label(text="Dialog open...", icon='INFO')
             elif status:
                 col.label(text=f"Status: {status}", icon='INFO')
             else:
                 col.label(text="Operation active...", icon='INFO')

classes = (
    ScaleInteractiveSettings, # Add settings class
    OBJECT_OT_ScaleToDimensionInteractive,
    OBJECT_OT_ScaleConfirmDialog,
    VIEW3D_PT_ScaleToDimensionInteractivePanel,
)
def register():
    handle_to_remove = modal_data.get("draw_handle")
    if handle_to_remove is not None:
        try:
            bpy.types.SpaceView3D.draw_handler_remove(handle_to_remove, 'WINDOW')
        except ValueError: pass
        except Exception: pass
    modal_data.clear()
    for cls in classes:
        bpy.utils.register_class(cls)
    # Add pointer property to Scene
    bpy.types.Scene.scale_interactive_settings = PointerProperty(type=ScaleInteractiveSettings)

def unregister():
    # Remove pointer property from Scene first
    if hasattr(bpy.types.Scene, 'scale_interactive_settings'):
        del bpy.types.Scene.scale_interactive_settings

    handle_to_remove = modal_data.get("draw_handle")
    if handle_to_remove is not None:
        try:
            bpy.types.SpaceView3D.draw_handler_remove(handle_to_remove, 'WINDOW')
        except ValueError: pass
        except Exception: pass
    modal_data.clear()
    for cls in reversed(classes):
        try:
            bpy.utils.unregister_class(cls)
        except RuntimeError:
            pass
if __name__ == "__main__":
    try:
        unregister()
    except Exception as e:
        print(f"Pre-registration unregister failed (might be first run): {e}")
    register()
